//## begin module%1.4%.codegen_version preserve=yes
//   Read the documentation to learn more about C++ code generator
//   versioning.
//## end module%1.4%.codegen_version

//## begin module%4A9C47F7020D.cm preserve=no
//	  %X% %Q% %Z% %W%
//## end module%4A9C47F7020D.cm

//## begin module%4A9C47F7020D.cp preserve=no
//## end module%4A9C47F7020D.cp

//## Module: PlotDirector%4A9C47F7020D; Package specification
//## Subsystem: XSPlot::Plot%3D625E94026B
//## Source file: /local/data/xerxes1/gordon/headas/Xspec/src/XSPlot/Plot/PlotDirector.h

#ifndef PLOTDIRECTOR_H
#define PLOTDIRECTOR_H 1

//## begin module%4A9C47F7020D.additionalIncludes preserve=no
//## end module%4A9C47F7020D.additionalIncludes

//## begin module%4A9C47F7020D.includes preserve=yes
//## end module%4A9C47F7020D.includes

// PlotSettings
#include <XSPlot/Plot/PlotSettings.h>

class PlotPkg;
class PlotCommand;

//## begin module%4A9C47F7020D.declarations preserve=no
//## end module%4A9C47F7020D.declarations

//## begin module%4A9C47F7020D.additionalDeclarations preserve=yes
#include <xsTypes.h>
#include <XSPlot/Plot/PlotTypes.h>
#include <XSUtil/Utils/XSutility.h>
//## end module%4A9C47F7020D.additionalDeclarations


//## begin PlotDirector%4A9C48D3005B.preface preserve=yes
//## end PlotDirector%4A9C48D3005B.preface

//## Class: PlotDirector%4A9C48D3005B
//	Singleton class to perform the central role in gathering
//	data and displaying a plot.  It provides a central
//	location for storing the PlotSettings object and the
//	container of PlotGroups, and reduces the coupling among
//	the various plot classes (ie. Commands, Panes,
//	Packages).  It is not quite the classic GoF Mediator
//	pattern since it doesn't maintain 2-way communication
//	with a set of related Colleague classes (excepting Plot
//	Command), but it serves a similar purpose.
//## Category: XSPlot::Plot%3D625E6A0065
//## Subsystem: XSPlot::Plot%3D625E94026B
//## Persistence: Transient
//## Cardinality/Multiplicity: n



class PlotDirector 
{
  //## begin PlotDirector%4A9C48D3005B.initialDeclarations preserve=yes
  //## end PlotDirector%4A9C48D3005B.initialDeclarations

  public:
    //## Class: UnitsContainer%4AFD7FD80073
    //	Plot units information:  The map key string is the
    //	identifier the user enters through setplot.  The value
    //	pair is composed of a string containing the unit's plot
    //	label appearance, and a Real for its conversion factor.
    //## Category: XSPlot::Plot%3D625E6A0065
    //## Subsystem: XSPlot::Plot%3D625E94026B
    //## Persistence: Transient
    //## Cardinality/Multiplicity: n



    typedef std::map<string,std::pair<string,Real>,XSutility::Nocase> UnitsContainer;

    //## Destructor (generated)
      ~PlotDirector();


    //## Other Operations (specified)
      //## Operation: Instance%4A9C4BBE017B
      static PlotDirector* Instance (const string& plottingPackage, const string& labelDictionaryPath);

      //## Operation: makePlot%4A9D33370215
      //	This is the one-step public function for executing a
      //	plot.  It converts the requested PlotCommands into Plot
      //	Pages, and then displays them.
      void makePlot (const std::vector<PlotCommand*>& commands);

      //## Operation: setplot%4AC6749D00F4
      PlotSettings& setplot ();

      //## Operation: setPlottingDevice%4AC67E4B032E
      //	Note that plotting device get and set functions are
      //	included here rather than PlotSettings since this
      //	setting must be immediately forwarded to the PlotPkg
      //	class.  It can't wait for makePlot to be called.
      void setPlottingDevice (const string& device, bool splashOn = true);

      //## Operation: getPlottingDeviceName%4ACA09620160
      const string& getPlottingDeviceName ();

      //## Operation: selectUnits%4AFDBAAB0078
      //	Sets PlotSetting's energy and wave unit information
      //	based on a case-insensitive match of the unitID string.
      //	Throws if no match is found. Mode should either be
      //	ENERGY or WAVELENGTH
      void selectUnits (const string& unitID, XaxisMode mode);

    //## Get and Set Operations for Class Attributes (generated)

      //## Attribute: lastCommand%4AC6702300F1
      const StringArray& lastCommand () const;
      void lastCommand (const StringArray& value);

      //## Attribute: lastCommandIndices%4B4CA9130242
      //	Stores the argument positions of the strings stored in
      //	lastCommand.  As per standard Xspec command-line comma
      //	usage, these need not be consecutive.
      const IntegerArray& lastCommandIndices () const;
      void lastCommandIndices (const IntegerArray& value);

      //## Attribute: savedPlotArray%4ACA1A4302FF
      //	For accessing the results of a tclout plot call.
      const std::vector<Real>& savedPlotArray () const;

      //## Attribute: MAX_IN_STACK%4AF44B15038C
      //	Hardcoding 2 plot features:
      //	1.  Max number of panes in a vertical stack.
      //	2.  Max number of vertical stacks.
      //	Perhaps in future, these should be attributes of
      //	particular PlotPkg classes, but PlotDirector is using
      //	this to determine whether x-axis is shared amongst panes
      //	in a stack.
      static size_t MAX_IN_STACK ();

      //## Attribute: MAX_STACKS%4AF44C210088
      static size_t MAX_STACKS ();

    //## Get and Set Operations for Associations (generated)

      //## Association: XSPlot::Plot::<unnamed>%4A9D5AFB03E4
      //## Role: PlotDirector::setplot%4A9D5AFD0005
      const PlotSettings& setplot () const;

  public:
    // Additional Public Declarations
      //## begin PlotDirector%4A9C48D3005B.public preserve=yes
      //## end PlotDirector%4A9C48D3005B.public

  protected:
    // Additional Protected Declarations
      //## begin PlotDirector%4A9C48D3005B.protected preserve=yes
      //## end PlotDirector%4A9C48D3005B.protected

  private:
    //## Constructors (generated)
      PlotDirector(const PlotDirector &right);

    //## Constructors (specified)
      //## Operation: PlotDirector%4A9C4AD3037D
      PlotDirector (const string& plottingPackage, const string& labelDictionaryPath);

    //## Assignment Operation (generated)
      PlotDirector & operator=(const PlotDirector &right);


    //## Other Operations (specified)
      //## Operation: loadLabelDictionary%4AA0281D019C
      void loadLabelDictionary (const string& dictionaryPath);

      //## Operation: loadUnitsInformation%4AFD86BB03C9
      void loadUnitsInformation (const string& path);

      //## Operation: clearPlotGroups%4ACCE98200E5
      //## Exceptions: ()
      void clearPlotGroups () throw ();

      //## Operation: fillPlotVectorAttributes%4AE09C2C035C
      void fillPlotVectorAttributes (size_t iCmd);

      //## Operation: determinePanesPerStack%4AF481E80248
      void determinePanesPerStack (std::vector<size_t>& panesPerStack) const;

      //## Operation: determineSharedXaxis%4AF482520165
      void determineSharedXaxis (const std::vector<size_t>& panesPerStack, std::vector<bool>& shareX) const;

      //## Operation: setPlottingPackage%4B1D5A0C0396
      void setPlottingPackage (const string& packageName);

      //## Operation: saveTcloutArray%4B58D2060121
      void saveTcloutArray ();

      //## Operation: buildLineIDList%4B5F5FD6006C
      void buildLineIDList (const PlotRange& plotRange, LineIDContainer& lineIDs) const;

    // Additional Private Declarations
      //## begin PlotDirector%4A9C48D3005B.private preserve=yes
      //## end PlotDirector%4A9C48D3005B.private

  private: //## implementation
    // Data Members for Class Attributes

      //## Attribute: instance%4A9C4B2F038B
      //## begin PlotDirector::instance%4A9C4B2F038B.attr preserve=no  private: static PlotDirector* {U} 0
      static PlotDirector* s_instance;
      //## end PlotDirector::instance%4A9C4B2F038B.attr

      //## Attribute: labelDictionaryFile%4AA028B10091
      //## begin PlotDirector::labelDictionaryFile%4AA028B10091.attr preserve=no  private: string {UC} 
      const string m_labelDictionaryFile;
      //## end PlotDirector::labelDictionaryFile%4AA028B10091.attr

      //## Attribute: unitsInformationFile%4AFD84C301D1
      //	Assume this is in the same directory as the label
      //	dictionary  file.  Therefore it doesn't need its own
      //	path specifier.
      //## begin PlotDirector::unitsInformationFile%4AFD84C301D1.attr preserve=no  private: string {UC} 
      const string m_unitsInformationFile;
      //## end PlotDirector::unitsInformationFile%4AFD84C301D1.attr

      //## Attribute: energyUnitsInformation%4AFD8646009C
      //## begin PlotDirector::energyUnitsInformation%4AFD8646009C.attr preserve=no  private: PlotDirector::UnitsContainer {U} 
      PlotDirector::UnitsContainer m_energyUnitsInformation;
      //## end PlotDirector::energyUnitsInformation%4AFD8646009C.attr

      //## Attribute: waveUnitsInformation%4B58B8440110
      //## begin PlotDirector::waveUnitsInformation%4B58B8440110.attr preserve=no  private: PlotDirector::UnitsContainer {U} 
      PlotDirector::UnitsContainer m_waveUnitsInformation;
      //## end PlotDirector::waveUnitsInformation%4B58B8440110.attr

      //## Attribute: plottingPackageName%4AA02BA2031D
      //## begin PlotDirector::plottingPackageName%4AA02BA2031D.attr preserve=no  private: string {U} 
      string m_plottingPackageName;
      //## end PlotDirector::plottingPackageName%4AA02BA2031D.attr

      //## begin PlotDirector::lastCommand%4AC6702300F1.attr preserve=no  public: StringArray {U} 
      StringArray m_lastCommand;
      //## end PlotDirector::lastCommand%4AC6702300F1.attr

      //## begin PlotDirector::lastCommandIndices%4B4CA9130242.attr preserve=no  public: IntegerArray {U} 
      IntegerArray m_lastCommandIndices;
      //## end PlotDirector::lastCommandIndices%4B4CA9130242.attr

      //## begin PlotDirector::savedPlotArray%4ACA1A4302FF.attr preserve=no  public: std::vector<Real> {U} 
      std::vector<Real> m_savedPlotArray;
      //## end PlotDirector::savedPlotArray%4ACA1A4302FF.attr

      //## Attribute: groups%4ACCE4F50256
      //	Stores PlotGroups, collated by 0-based PlotPane index.
      //## begin PlotDirector::groups%4ACCE4F50256.attr preserve=no  private: PlotGroupContainer {U} 
      PlotGroupContainer m_groups;
      //## end PlotDirector::groups%4ACCE4F50256.attr

      //## begin PlotDirector::MAX_IN_STACK%4AF44B15038C.attr preserve=no  public: static size_t {UC} 3
      static const size_t s_MAX_IN_STACK;
      //## end PlotDirector::MAX_IN_STACK%4AF44B15038C.attr

      //## begin PlotDirector::MAX_STACKS%4AF44C210088.attr preserve=no  public: static size_t {UC} 2
      static const size_t s_MAX_STACKS;
      //## end PlotDirector::MAX_STACKS%4AF44C210088.attr

    // Data Members for Associations

      //## Association: XSPlot::Plot::<unnamed>%4A9D5AFB03E4
      //## begin PlotDirector::setplot%4A9D5AFD0005.role preserve=no  public: PlotSettings { -> 1VHgN}
      PlotSettings m_setplot;
      //## end PlotDirector::setplot%4A9D5AFD0005.role

      //## Association: XSPlot::Plot::<unnamed>%4AB397CA02E2
      //## Role: PlotDirector::plotPackage%4AB397CC00BF
      //## begin PlotDirector::plotPackage%4AB397CC00BF.role preserve=no  public: PlotPkg { -> 1RFHgN}
      PlotPkg* m_plotPackage;
      //## end PlotDirector::plotPackage%4AB397CC00BF.role

      //## Association: XSPlot::Plot::<unnamed>%4AB7BAD602BD
      //## Role: PlotDirector::plotCommands%4AB7BAD7025C
      //## begin PlotDirector::plotCommands%4AB7BAD7025C.role preserve=no  public: PlotCommand { -> 1..nRFHgN}
      std::vector<PlotCommand*> m_plotCommands;
      //## end PlotDirector::plotCommands%4AB7BAD7025C.role

    // Additional Implementation Declarations
      //## begin PlotDirector%4A9C48D3005B.implementation preserve=yes
      //## end PlotDirector%4A9C48D3005B.implementation

};

//## begin PlotDirector%4A9C48D3005B.postscript preserve=yes
//## end PlotDirector%4A9C48D3005B.postscript

// Class PlotDirector 


//## Other Operations (inline)
inline PlotSettings& PlotDirector::setplot ()
{
  //## begin PlotDirector::setplot%4AC6749D00F4.body preserve=yes
   return m_setplot;
  //## end PlotDirector::setplot%4AC6749D00F4.body
}

//## Get and Set Operations for Class Attributes (inline)

inline const StringArray& PlotDirector::lastCommand () const
{
  //## begin PlotDirector::lastCommand%4AC6702300F1.get preserve=no
  return m_lastCommand;
  //## end PlotDirector::lastCommand%4AC6702300F1.get
}

inline void PlotDirector::lastCommand (const StringArray& value)
{
  //## begin PlotDirector::lastCommand%4AC6702300F1.set preserve=no
  m_lastCommand = value;
  //## end PlotDirector::lastCommand%4AC6702300F1.set
}

inline const IntegerArray& PlotDirector::lastCommandIndices () const
{
  //## begin PlotDirector::lastCommandIndices%4B4CA9130242.get preserve=no
  return m_lastCommandIndices;
  //## end PlotDirector::lastCommandIndices%4B4CA9130242.get
}

inline void PlotDirector::lastCommandIndices (const IntegerArray& value)
{
  //## begin PlotDirector::lastCommandIndices%4B4CA9130242.set preserve=no
  m_lastCommandIndices = value;
  //## end PlotDirector::lastCommandIndices%4B4CA9130242.set
}

inline const std::vector<Real>& PlotDirector::savedPlotArray () const
{
  //## begin PlotDirector::savedPlotArray%4ACA1A4302FF.get preserve=no
  return m_savedPlotArray;
  //## end PlotDirector::savedPlotArray%4ACA1A4302FF.get
}

inline size_t PlotDirector::MAX_IN_STACK ()
{
  //## begin PlotDirector::MAX_IN_STACK%4AF44B15038C.get preserve=no
  return s_MAX_IN_STACK;
  //## end PlotDirector::MAX_IN_STACK%4AF44B15038C.get
}

inline size_t PlotDirector::MAX_STACKS ()
{
  //## begin PlotDirector::MAX_STACKS%4AF44C210088.get preserve=no
  return s_MAX_STACKS;
  //## end PlotDirector::MAX_STACKS%4AF44C210088.get
}

//## Get and Set Operations for Associations (inline)

inline const PlotSettings& PlotDirector::setplot () const
{
  //## begin PlotDirector::setplot%4A9D5AFD0005.get preserve=no
  return m_setplot;
  //## end PlotDirector::setplot%4A9D5AFD0005.get
}

//## begin module%4A9C47F7020D.epilog preserve=yes
//## end module%4A9C47F7020D.epilog


#endif
